----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:59:06 06/30/2012 
-- Design Name: 
-- Module Name:    quad_decoder - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity quad_decoder is
		port (
			clk: in std_logic;
			reset: in std_logic;
			A, B       : in  std_logic;
			up, down :out std_logic
	);
end quad_decoder;

architecture Behavioral of quad_decoder is
	--3 stage A&B sampling (2 for stability, 1 for transition analysis)
	signal A_delayed, B_delayed : std_logic_vector(2 downto 0);

	signal up_int, down_int : std_logic;

	signal count_enable : std_logic; -- signal that clocks counts
	signal count_invalid : std_logic; -- signal that clocks invalid transitions
	signal count_direction : std_logic; -- signal that has direction during counts
begin
	up <= up_int;
	down <= down_int;

	count_enable <= A_delayed(1) XOR A_delayed(2) XOR B_delayed(1) XOR B_delayed(2); --A changed or B changed, but not both (both would be invalid).
	count_invalid <= (A_delayed(1) XOR A_delayed(2)) AND (B_delayed(1) XOR B_delayed(2)); --A changed AND B changed (invalid).
	count_direction <= A_delayed(1) XOR B_delayed(2);

	CaptureInputProcess : process (clk, reset)
	begin
		if reset = '1' then --Reset
			A_delayed <= (others => A); 
			B_delayed <= (others => B); 
		elsif  rising_edge(clk)  then --Clock
			A_delayed <= A_delayed(1 downto 0) & A;
			B_delayed <= B_delayed(1 downto 0) & B;
		end if;
	end process;

	CountProcess: process (clk, reset)
	begin
		if reset = '1' then
			up_int <= '0';
			down_int <= '0';
--			counter_Int <= (others => '0'); 
--			counterInvalid_Int <= (others => '0'); 
		elsif  rising_edge(clk)  then --Clock
			up_int <= '0';
			down_int <= '0';
			if  (count_invalid='1')  then -- Count invalid
				--counterInvalid_Int <= counterInvalid_Int + 1;
			elsif (count_enable='1')  then -- Count
				if (count_direction='1')  then -- Count Up
					up_int <= '1';
				else -- Count Down
					down_int <= '1';
				end if;
			end if;
		end if;
	end process;

end Behavioral;

