library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use ieee.numeric_std.all;


library work;
use work.zpu_config.all;
use work.zpupkg.all;
use work.zpuinopkg.all;

entity zpuino_quadcounter is
		generic ( 
			COUNTER_BIT_WIDTH : integer := wordSize;
			COUNTER_INVALID_BIT_WIDTH : integer := wordSize;
			CHANNELS : integer := 1
			);
		port (
			-- Wishbone Bus
			wb_clk_i: in std_logic;
			wb_rst_i: in std_logic;
			wb_dat_o: out std_logic_vector(wordSize-1 downto 0);
			wb_dat_i: in std_logic_vector(wordSize-1 downto 0);
			wb_adr_i: in std_logic_vector(maxIObit downto minIObit);
			wb_we_i:  in std_logic;
			wb_cyc_i: in std_logic;
			wb_stb_i: in std_logic;
			wb_ack_o: out std_logic;
			wb_inta_o:out std_logic;

			-- Counter Input
			A, B       : in  std_logic_vector(CHANNELS-1 downto 0)
	);
end zpuino_quadcounter;

     
architecture behave of zpuino_quadcounter is

	type tCounterArray is array (CHANNELS-1 downto 0) of std_logic_vector(COUNTER_BIT_WIDTH-1 downto 0);
	type tCounterInvalidArray is array (CHANNELS-1 downto 0) of std_logic_vector(COUNTER_INVALID_BIT_WIDTH-1 downto 0);

	signal counter_Int : tCounterArray; --Internal counters
	signal counterInvalid_Int: tCounterInvalidArray; --Internal invalid counters


	--signal wb_dat_o_Int  : std_logic_vector(wb_dat_o'range);
	
	signal up,down : std_logic_vector(CHANNELS-1 downto 0);
	
COMPONENT quad_decoder
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		A : IN std_logic;
		B : IN std_logic;          
		up : OUT std_logic;
		down : OUT std_logic
		);
	END COMPONENT;	

begin
	--wb_dat_o <= wb_dat_o_Int;
	--counter <= counter_Int;
	--counterInvalid <= counterInvalid_Int;

	wb_ack_o <= wb_cyc_i and wb_stb_i; --Acknowledge any read


	channel_inst : for i in 0 to CHANNELS-1 generate

		quad_decoder_inst: quad_decoder
			port map (
				clk   => wb_clk_i,
				reset   => wb_rst_i,
				A=> A(i),
				B => B(i),
				up => up(i),
				down => down(i)
			);

		CountProcess: process (wb_clk_i, wb_rst_i)
		begin
			if wb_rst_i = '1' then
				counter_Int(i) <= (others => '0'); 
	--			counterInvalid_Int <= (others => '0'); 
			elsif rising_edge(wb_clk_i)  then --Clock
				if (up(i)='1')  then -- Count Up
					counter_Int(i) <= counter_Int(i) + '1';
				elsif (down(i)='1') then -- Count Down
					counter_Int(i) <= counter_Int(i) - '1';
				end if;
			end if;
		end process;
	  
	end generate;

	LoadToDataOutBus:process(wb_adr_i, counter_Int) --wb_clk_i, wb_rst_i, 
	begin
		wb_dat_o <= (others=>'0');
		for i in 0 to CHANNELS-1 loop
			if (wb_adr_i(15 downto 2) = std_logic_vector(to_unsigned(i, wb_adr_i'length)) ) then
				wb_dat_o <= counter_Int( i );
			end if;
		end loop;
	end process;

end behave;

